#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_switch_stacks_routing_static_routes
short_description: Resource module for networks _switch _stacks _routing _staticroutes
description:
- Manage operations create, update and delete of the resource networks _switch _stacks _routing _staticroutes.
- Create a layer 3 static route for a switch stack.
- Delete a layer 3 static route for a switch stack.
- Update a layer 3 static route for a switch stack.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  advertiseViaOspfEnabled:
    description: Option to advertise static route via OSPF.
    type: bool
  name:
    description: Name or description for layer 3 static route.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  nextHopIp:
    description: IP address of the next hop device to which the device sends its traffic
      for the subnet.
    type: str
  preferOverOspfRoutesEnabled:
    description: Option to prefer static route over OSPF routes.
    type: bool
  staticRouteId:
    description: StaticRouteId path parameter. Static route ID.
    type: str
  subnet:
    description: The subnet which is routed via this static route and should be specified
      in CIDR notation (ex. 1.2.3.0/24).
    type: str
  switchStackId:
    description: SwitchStackId path parameter. Switch stack ID.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for switch createNetworkSwitchStackRoutingStaticRoute
  description: Complete reference of the createNetworkSwitchStackRoutingStaticRoute API.
  link: https://developer.cisco.com/meraki/api-v1/#!create-network-switch-stack-routing-static-route
- name: Cisco Meraki documentation for switch deleteNetworkSwitchStackRoutingStaticRoute
  description: Complete reference of the deleteNetworkSwitchStackRoutingStaticRoute API.
  link: https://developer.cisco.com/meraki/api-v1/#!delete-network-switch-stack-routing-static-route
- name: Cisco Meraki documentation for switch updateNetworkSwitchStackRoutingStaticRoute
  description: Complete reference of the updateNetworkSwitchStackRoutingStaticRoute API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network-switch-stack-routing-static-route
notes:
  - SDK Method used are
    switch.Switch.create_network_switch_stack_routing_static_route,
    switch.Switch.delete_network_switch_stack_routing_static_route,
    switch.Switch.update_network_switch_stack_routing_static_route,

  - Paths used are
    post /networks/{networkId}/switch/stacks/{switchStackId}/routing/staticRoutes,
    delete /networks/{networkId}/switch/stacks/{switchStackId}/routing/staticRoutes/{staticRouteId},
    put /networks/{networkId}/switch/stacks/{switchStackId}/routing/staticRoutes/{staticRouteId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_switch_stacks_routing_static_routes:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    advertiseViaOspfEnabled: false
    name: My route
    networkId: string
    nextHopIp: 1.2.3.4
    preferOverOspfRoutesEnabled: false
    subnet: 192.168.1.0/24
    switchStackId: string

- name: Update by id
  cisco.meraki.networks_switch_stacks_routing_static_routes:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    advertiseViaOspfEnabled: false
    name: My route
    networkId: string
    nextHopIp: 1.2.3.4
    preferOverOspfRoutesEnabled: false
    staticRouteId: string
    subnet: 192.168.1.0/24
    switchStackId: string

- name: Delete by id
  cisco.meraki.networks_switch_stacks_routing_static_routes:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: absent
    networkId: string
    staticRouteId: string
    switchStackId: string

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""
