# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function


__metaclass__ = type

# Copyright: (c) 2015, Peter Sprygada <psprygada@ansible.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


class ModuleDocFragment(object):
    # Standard files documentation fragment
    DOCUMENTATION = r"""options: {}
notes:
  - For information on using CLI and NX-API see the :ref:`NXOS Platform Options guide
    <nxos_platform_options>`
  - For more information on using Ansible to manage network devices see the :ref:`Ansible
    Network Guide <network_guide>`
  - For more information on using Ansible to manage Cisco devices see the `Cisco integration
    page <https://www.ansible.com/integrations/networks/cisco>`_.
"""
